@@ -36,6 +36,8 @@
 #include "prng.h"
 #include "options.h"
 #include "logging.h"
+#include "create_pdf.h"
+#include "miscellaneous.h"
 
 /* Global array to hold log values to print when logging to STDOUT */
 char** log_lines;
@@ -84,6 +86,19 @@ void nwipe_log( nwipe_log_t level, const char* format, ... )
     /* initialise characters written */
     chars_written = 0;
 
+    /* Only log messages with the debug label if the command line --verbose
+     * options has been specified, otherwise just return
+     */
+    if( level == NWIPE_LOG_DEBUG && nwipe_options.verbose == 0 )
+    {
+        r = pthread_mutex_unlock( &mutex1 );
+        if( r != 0 )
+        {
+            fprintf( stderr, "nwipe_log: pthread_mutex_unlock failed. Code %i \n", r );
+        }
+        return;
+    }
+
     /* Print the date. The rc script uses the same format. */
     if( level != NWIPE_LOG_NOTIMESTAMP )
     {
@@ -386,7 +401,6 @@ void nwipe_log( nwipe_log_t level, const char* format, ... )
             }
             user_abort = 1;
             terminate_signal = 1;
-            return;
         }
     }
 
@@ -405,9 +419,6 @@ void nwipe_perror( int nwipe_errno, const char* f, const char* s )
 {
     /**
      * Wrapper for perror().
-     *
-     * We may wish to tweak or squelch this later.
-     *
      */
 
     nwipe_log( NWIPE_LOG_ERROR, "%s: %s: %s", f, s, strerror( nwipe_errno ) );
@@ -603,16 +614,16 @@ int nwipe_log_sysinfo()
                     if( *( &dmidecode_keywords[keywords_idx][1][0] ) == '0' )
                     {
                         nwipe_log(
-                            NWIPE_LOG_NOTICE, "%s = %s", &dmidecode_keywords[keywords_idx][0][0], "XXXXXXXXXXXXXXX" );
+                            NWIPE_LOG_INFO, "%s = %s", &dmidecode_keywords[keywords_idx][0][0], "XXXXXXXXXXXXXXX" );
                     }
                     else
                     {
-                        nwipe_log( NWIPE_LOG_NOTICE, "%s = %s", &dmidecode_keywords[keywords_idx][0][0], path );
+                        nwipe_log( NWIPE_LOG_INFO, "%s = %s", &dmidecode_keywords[keywords_idx][0][0], path );
                     }
                 }
                 else
                 {
-                    nwipe_log( NWIPE_LOG_NOTICE, "%s = %s", &dmidecode_keywords[keywords_idx][0][0], path );
+                    nwipe_log( NWIPE_LOG_INFO, "%s = %s", &dmidecode_keywords[keywords_idx][0][0], path );
                 }
             }
             /* close */
@@ -635,7 +646,12 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
 {
     /* Prints two summary tables, the first is the device pass and verification summary
      * and the second is the main summary table detaining the drives, status, throughput,
-     * model and serial number */
+     * model and serial number.
+     *
+     * This function also calls the create_pdf() function that creates the PDF erasure
+     * report file. A page report on the success or failure of the erasure operation
+     */
+
     int i;
     int idx_src;
     int idx_dest;
@@ -650,7 +666,7 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
     // char duration[5];
     char duration[314];
     char model[18];
-    char serial_no[20];
+    char serial_no[NWIPE_SERIALNUMBER_LENGTH + 1];
     char exclamation_flag[2];
     int hours;
     int minutes;
@@ -775,26 +791,32 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
 
             strncpy( status, "-FAILED-", 8 );
             status[8] = 0;
+
+            strcpy( c[i]->wipe_status_txt, "FAILED" );  // copy to context for use by certificate
         }
         else
         {
-            if( c[i]->wipe_status == 0 )
+            if( c[i]->wipe_status == 0 /* && user_abort != 1 */ )
             {
                 strncpy( exclamation_flag, " ", 1 );
                 exclamation_flag[1] = 0;
 
                 strncpy( status, " Erased ", 8 );
                 status[8] = 0;
+
+                strcpy( c[i]->wipe_status_txt, "ERASED" );  // copy to context for use by certificate
             }
             else
             {
-                if( user_abort == 1 )
+                if( c[i]->wipe_status == 1 && user_abort == 1 )
                 {
                     strncpy( exclamation_flag, "!", 1 );
                     exclamation_flag[1] = 0;
 
                     strncpy( status, "UABORTED", 8 );
                     status[8] = 0;
+
+                    strcpy( c[i]->wipe_status_txt, "ABORTED" );  // copy to context for use by certificate
                 }
                 else
                 {
@@ -804,6 +826,8 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
 
                     strncpy( status, "INSANITY", 8 );
                     status[8] = 0;
+
+                    strcpy( c[i]->wipe_status_txt, "INSANITY" );  // copy to context for use by certificate
                 }
             }
         }
@@ -811,6 +835,9 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
         /* Determine the size of throughput so that the correct nomenclature can be used */
         Determine_C_B_nomenclature( c[i]->throughput, throughput, 13 );
 
+        /* write the duration string to the drive context for later use by create_pdf() */
+        snprintf( c[i]->throughput_txt, sizeof( c[i]->throughput_txt ), "%s", throughput );
+
         /* Add this devices throughput to the total throughput */
         total_throughput += c[i]->throughput;
 
@@ -825,8 +852,13 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
         {
             if( c[i]->start_time != 0 && c[i]->end_time == 0 )
             {
-                /* For a summary in the event of a system shutdown */
+                /* For a summary in the event of a system shutdown, user abort */
                 c[i]->duration = difftime( t, c[i]->start_time );
+
+                /* If end_time is zero, which may occur if the wipe is aborted, then set
+                 * end_time to current time. Important to do as endtime is used by
+                 * the PDF report function */
+                c[i]->end_time = time( &t );
             }
         }
 
@@ -835,12 +867,16 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
         /* Convert binary seconds into three binary variables, hours, minutes and seconds */
         convert_seconds_to_hours_minutes_seconds( total_duration_seconds, &hours, &minutes, &seconds );
 
+        /* write the duration string to the drive context for later use by create_pdf() */
+        snprintf( c[i]->duration_str, sizeof( c[i]->duration_str ), "%02i:%02i:%02i", hours, minutes, seconds );
+
         /* Device Model */
         strncpy( model, c[i]->device_model, 17 );
         model[17] = 0;
 
         /* Serial No. */
-        strncpy( serial_no, c[i]->device_serial_no, 20 );
+        strncpy( serial_no, c[i]->device_serial_no, NWIPE_SERIALNUMBER_LENGTH );
+        serial_no[NWIPE_SERIALNUMBER_LENGTH] = 0;
         model[17] = 0;
 
         nwipe_log( NWIPE_LOG_NOTIMESTAMP,
@@ -854,6 +890,15 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
                    seconds,
                    model,
                    serial_no );
+
+        /* Create the PDF report/certificate */
+        if( nwipe_options.PDF_enable == 1 )
+        // if( strcmp( nwipe_options.PDFreportpath, "noPDF" ) != 0 )
+        {
+            /* to have some progress indication. can help if there are many/slow disks */
+            fprintf( stderr, "." );
+            create_pdf( c[i] );
+        }
     }
 
     /* Determine the size of throughput so that the correct nomenclature can be used */
@@ -903,105 +948,12 @@ void nwipe_log_summary( nwipe_context_t** ptr, int nwipe_selected )
     nwipe_log( NWIPE_LOG_NOTIMESTAMP,
                "********************************************************************************" );
     nwipe_log( NWIPE_LOG_NOTIMESTAMP, "" );
-}
 
-void Determine_C_B_nomenclature( u64 speed, char* result, int result_array_size )
-{
-
-    /* C_B ? Determine Capacity or Bandwidth nomenclature
-     *
-     * A pointer to a result character string with a minimum of 13 characters in length
-     * should be provided.
-     *
-     * Outputs a string of the form xxxTB/s, xxxGB/s, xxxMB/s, xxxKB/s B/s depending on the value of 'speed'
+    /* Log information regarding where the PDF certificate is saved but log after the summary table so
+     * this information is only printed once.
      */
-
-    /* Initialise the output array */
-    int idx = 0;
-
-    while( idx < result_array_size )
+    if( strcmp( nwipe_options.PDFreportpath, "noPDF" ) != 0 )
     {
-        result[idx++] = 0;
-    }
-
-    /* Determine the size of throughput so that the correct nomenclature can be used */
-    if( speed >= INT64_C( 1000000000000 ) )
-    {
-        snprintf( result, result_array_size, "%3llu TB", speed / INT64_C( 1000000000000 ) );
-    }
-    else if( speed >= INT64_C( 1000000000 ) )
-    {
-        snprintf( result, result_array_size, "%3llu GB", speed / INT64_C( 1000000000 ) );
-    }
-    else if( speed >= INT64_C( 1000000 ) )
-    {
-        snprintf( result, result_array_size, "%3llu MB", speed / INT64_C( 1000000 ) );
-    }
-    else if( speed >= INT64_C( 1000 ) )
-    {
-        snprintf( result, result_array_size, "%3llu KB", speed / INT64_C( 1000 ) );
-    }
-    else
-    {
-        snprintf( result, result_array_size, "%3llu B", speed / INT64_C( 1 ) );
-    }
-}
-
-void convert_seconds_to_hours_minutes_seconds( u64 total_seconds, int* hours, int* minutes, int* seconds )
-{
-    /* Convert binary seconds into binary hours, minutes and seconds */
-
-    if( total_seconds % 60 )
-    {
-        *minutes = total_seconds / 60;
-
-        *seconds = total_seconds - ( *minutes * 60 );
-    }
-    else
-    {
-        *minutes = total_seconds / 60;
-
-        *seconds = 0;
-    }
-    if( *minutes > 59 )
-    {
-        *hours = *minutes / 60;
-        if( *minutes % 60 )
-        {
-            *minutes = *minutes - ( *hours * 60 );
-        }
-        else
-        {
-            *minutes = 0;
-        }
-    }
-}
-
-int nwipe_strip_path( char* output, char* input )
-{
-    /* Take the input string, say "/dev/sda" and remove the "/dev/", prefix the result
-     * with 'length' spaces. So if length=8 and input=/dev/sda, output will
-     * be "     sda", a string 8 characters long right justified with spaces.
-     */
-    int idx_dest;
-    int idx_src;
-    idx_dest = 8;
-    // idx_dest = length;
-    output[idx_dest--] = 0;
-    idx_src = strlen( input );
-    idx_src--;
-
-    while( idx_dest >= 0 )
-    {
-        /* if the device name contains a / start prefixing spaces */
-        if( input[idx_src] == '/' )
-        {
-            output[idx_dest--] = ' ';
-            continue;
-        }
-        if( idx_src >= 0 )
-        {
-            output[idx_dest--] = input[idx_src--];
-        }
+        nwipe_log( NWIPE_LOG_NOTIMESTAMP, "Creating PDF report in %s\n", nwipe_options.PDFreportpath );
     }
 }
