@@ -629,25 +629,81 @@ void fix_endian_model_names( char* model )
     int idx2 = 0;
     unsigned int length = 0;
     char* tmp_string;
+    char* model_lower_case;
+    int swap_endian_flag = 0;
 
     length = strlen( model );
 
-    tmp_string = calloc( length, 1 );
+    tmp_string = calloc( length + 1, 1 );
+    model_lower_case = calloc( length + 1, 1 );
+
+    strncpy( model_lower_case, model, length );
+    model_lower_case[length] = 0; /* makesure it's terminated */
+    strlower( model_lower_case ); /* convert to lower case for comparison */
 
     /* "ASSMNU G" = "SAMSUNG ", tested against model Samsung HM160HC so that
      * "ASSMNU G MH61H0 C" becomes "SAMSUNG HM160HC ")
      */
-    if( !( strncmp( model, "ASSMNU G", 8 ) ) )
+
+    if( !( strncmp( model_lower_case, "assmnu g", 8 ) ) )
     {
-        while( model[idx] != 0 )
+        swap_endian_flag = 1;
+    }
+    else
+    {
+        /* Hitachi */
+        if( !( strncmp( model_lower_case, "ihathc i", 8 ) ) )
+        {
+            swap_endian_flag = 1;
+        }
+        else
+        {
+            /* Toshiba */
+            if( !( strncmp( model_lower_case, "othsbi a", 8 ) ) )
+            {
+                swap_endian_flag = 1;
+            }
+            else
+            {
+                /* WDC (Western Digital Corporation) */
+                if( !( strncmp( model_lower_case, "dw c", 4 ) ) )
+                {
+                    swap_endian_flag = 1;
+                }
+                else
+                {
+                    /* Seagate */
+                    if( !( strncmp( model_lower_case, "esgata e", 8 ) ) )
+                    {
+                        swap_endian_flag = 1;
+                    }
+                    else
+                    {
+                        /* Seagate models starting ST */
+                        if( !( strncmp( model_lower_case, "ts", 2 ) ) )
+                        {
+                            swap_endian_flag = 1;
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    if( swap_endian_flag == 1 )
+    {
+        while( model[idx] != 0 && idx < length )
         {
-            tmp_string[idx2 + 1] = model[idx];
             if( model[idx + 1] != 0 )
             {
+                /* Swap the bytes */
+                tmp_string[idx2 + 1] = model[idx];
                 tmp_string[idx2] = model[idx + 1];
             }
             else
             {
+                /* Copy the last odd byte and exit while loop */
+                tmp_string[idx2] = model[idx];
                 break;
             }
 
@@ -660,7 +716,10 @@ void fix_endian_model_names( char* model )
             idx2 += 2;
         }
 
-        tmp_string[idx2 + 1] = 0;
-        strcpy( model, tmp_string );
+        tmp_string[length] = 0; /* terminate */
+        strncpy( model, tmp_string, length );
+        model[length] = 0; /* terminate */
     }
+    free( tmp_string );
+    free( model_lower_case );
 }
