@@ -27,6 +27,7 @@
 #include "options.h"
 #include "logging.h"
 #include "version.h"
+#include "conf.h"
 
 /* The global options struct. */
 nwipe_options_t nwipe_options;
@@ -41,6 +42,8 @@ int nwipe_options_parse( int argc, char** argv )
     extern nwipe_prng_t nwipe_twister;
     extern nwipe_prng_t nwipe_isaac;
     extern nwipe_prng_t nwipe_isaac64;
+    extern nwipe_prng_t nwipe_add_lagg_fibonacci_prng;
+    extern nwipe_prng_t nwipe_xoroshiro256_prng;
 
     /* The getopt() result holder. */
     int nwipe_opt;
@@ -54,7 +57,12 @@ int nwipe_options_parse( int argc, char** argv )
     int i;
 
     /* The list of acceptable short options. */
-    char nwipe_options_short[] = "Vvhl:m:p:qr:e:";
+    char nwipe_options_short[] = "Vvhl:P:m:p:qr:e:";
+
+    /* Used when reading value fron nwipe.conf */
+    const char* read_value = NULL;
+
+    int ret;
 
     /* The list of acceptable long options. */
     static struct option nwipe_options_long[] = {
@@ -73,6 +81,9 @@ int nwipe_options_parse( int argc, char** argv )
         /* Log file. Corresponds to the 'l' short option. */
         { "logfile", required_argument, 0, 'l' },
 
+        /* PDFreport path. Corresponds to the 'P' short option. */
+        { "PDFreportpath", required_argument, 0, 'P' },
+
         /* Exclude devices, comma separated list */
         { "exclude", required_argument, 0, 'e' },
 
@@ -118,8 +129,9 @@ int nwipe_options_parse( int argc, char** argv )
     /* Set default options. */
     nwipe_options.autonuke = 0;
     nwipe_options.autopoweroff = 0;
-    nwipe_options.method = &nwipe_dodshort;
-    nwipe_options.prng = ( sizeof( unsigned long int ) >= 8 ) ? &nwipe_isaac64 : &nwipe_isaac;
+    nwipe_options.method = &nwipe_random;
+    nwipe_options.prng =
+        ( sizeof( unsigned long int ) >= 8 ) ? &nwipe_xoroshiro256_prng : &nwipe_add_lagg_fibonacci_prng;
     nwipe_options.rounds = 1;
     nwipe_options.noblank = 0;
     nwipe_options.nousb = 0;
@@ -131,6 +143,76 @@ int nwipe_options_parse( int argc, char** argv )
     nwipe_options.verbose = 0;
     nwipe_options.verify = NWIPE_VERIFY_LAST;
     memset( nwipe_options.logfile, '\0', sizeof( nwipe_options.logfile ) );
+    memset( nwipe_options.PDFreportpath, '\0', sizeof( nwipe_options.PDFreportpath ) );
+    strncpy( nwipe_options.PDFreportpath, ".", 2 );
+
+    /* Read PDF settings from nwipe.conf if available  */
+    if( ( ret = nwipe_conf_read_setting( "PDF_Certificate.PDF_Enable", &read_value ) ) )
+    {
+        /* error occurred */
+        nwipe_log( NWIPE_LOG_ERROR,
+                   "nwipe_conf_read_setting():Error reading PDF_Certificate.PDF_Enable from nwipe.conf, ret code %i",
+                   ret );
+
+        /* Use default values */
+        nwipe_options.PDF_enable = 1;
+    }
+    else
+    {
+        if( !strcmp( read_value, "ENABLED" ) )
+        {
+            nwipe_options.PDF_enable = 1;
+        }
+        else
+        {
+            if( !strcmp( read_value, "DISABLED" ) )
+            {
+                nwipe_options.PDF_enable = 0;
+            }
+            else
+            {
+                // error occurred
+                nwipe_log(
+                    NWIPE_LOG_ERROR,
+                    "PDF_Certificate.PDF_Enable in nwipe.conf returned a value that was neither ENABLED or DISABLED" );
+                nwipe_options.PDF_enable = 1;  // Default to Enabled
+            }
+        }
+    }
+
+    /* PDF Preview enable/disable */
+    if( ( ret = nwipe_conf_read_setting( "PDF_Certificate.PDF_Preview", &read_value ) ) )
+    {
+        /* error occurred */
+        nwipe_log( NWIPE_LOG_ERROR,
+                   "nwipe_conf_read_setting():Error reading PDF_Certificate.PDF_Preview from nwipe.conf, ret code %i",
+                   ret );
+
+        /* Use default values */
+        nwipe_options.PDF_enable = 1;
+    }
+    else
+    {
+        if( !strcmp( read_value, "ENABLED" ) )
+        {
+            nwipe_options.PDF_preview_details = 1;
+        }
+        else
+        {
+            if( !strcmp( read_value, "DISABLED" ) )
+            {
+                nwipe_options.PDF_preview_details = 0;
+            }
+            else
+            {
+                /* error occurred */
+                nwipe_log(
+                    NWIPE_LOG_ERROR,
+                    "PDF_Certificate.PDF_Preview in nwipe.conf returned a value that was neither ENABLED or DISABLED" );
+                nwipe_options.PDF_preview_details = 1; /* Default to Enabled */
+            }
+        }
+    }
 
     /* Initialise each of the strings in the excluded drives array */
     for( i = 0; i < MAX_NUMBER_EXCLUDED_DRIVES; i++ )
@@ -315,6 +397,29 @@ int nwipe_options_parse( int argc, char** argv )
                 strncpy( nwipe_options.logfile, optarg, sizeof( nwipe_options.logfile ) );
                 break;
 
+            case 'P': /* PDFreport path option. */
+
+                nwipe_options.PDFreportpath[strlen( optarg )] = '\0';
+                strncpy( nwipe_options.PDFreportpath, optarg, sizeof( nwipe_options.PDFreportpath ) );
+
+                /* Command line options will override what's in nwipe.conf */
+                if( strcmp( nwipe_options.PDFreportpath, "noPDF" ) == 0 )
+                {
+                    nwipe_options.PDF_enable = 0;
+                    nwipe_conf_update_setting( "PDF_Certificate.PDF_Enable", "DISABLED" );
+                }
+                else
+                {
+                    if( strcmp( nwipe_options.PDFreportpath, "." ) )
+                    {
+                        /* and if the user has specified a PDF path then enable PDF */
+                        nwipe_options.PDF_enable = 1;
+                        nwipe_conf_update_setting( "PDF_Certificate.PDF_Enable", "ENABLED" );
+                    }
+                }
+
+                break;
+
             case 'e': /* exclude drives option */
 
                 idx_drive_chr = 0;
@@ -354,6 +459,13 @@ int nwipe_options_parse( int argc, char** argv )
                             }
                         }
                     }
+                    if( idx_drive == MAX_NUMBER_EXCLUDED_DRIVES )
+                    {
+                        fprintf(
+                            stderr,
+                            "The number of excluded drives has reached the programs configured limit, aborting\n" );
+                        exit( 130 );
+                    }
                 }
                 break;
 
@@ -381,6 +493,16 @@ int nwipe_options_parse( int argc, char** argv )
                     nwipe_options.prng = &nwipe_isaac64;
                     break;
                 }
+                if( strcmp( optarg, "add_lagg_fibonacci_prng" ) == 0 )
+                {
+                    nwipe_options.prng = &nwipe_add_lagg_fibonacci_prng;
+                    break;
+                }
+                if( strcmp( optarg, "xoroshiro256_prng" ) == 0 )
+                {
+                    nwipe_options.prng = &nwipe_xoroshiro256_prng;
+                    break;
+                }
 
                 /* Else we do not know this PRNG. */
                 fprintf( stderr, "Error: Unknown prng '%s'.\n", optarg );
@@ -430,6 +552,8 @@ void nwipe_options_log( void )
     extern nwipe_prng_t nwipe_twister;
     extern nwipe_prng_t nwipe_isaac;
     extern nwipe_prng_t nwipe_isaac64;
+    extern nwipe_prng_t nwipe_add_lagg_fibonacci_prng;
+    extern nwipe_prng_t nwipe_xoroshiro256_prng;
 
     /**
      *  Prints a manifest of options to the log.
@@ -483,19 +607,33 @@ void nwipe_options_log( void )
     }
     else
     {
-        if( nwipe_options.prng == &nwipe_isaac )
+        if( nwipe_options.prng == &nwipe_add_lagg_fibonacci_prng )
         {
-            nwipe_log( NWIPE_LOG_NOTICE, "  prng     = Isaac" );
+            nwipe_log( NWIPE_LOG_NOTICE, "  prng     = Lagged Fibonacci generator (EXPERIMENTAL!)" );
         }
         else
         {
-            if( nwipe_options.prng == &nwipe_isaac64 )
+            if( nwipe_options.prng == &nwipe_xoroshiro256_prng )
             {
-                nwipe_log( NWIPE_LOG_NOTICE, "  prng     = Isaac64" );
+                nwipe_log( NWIPE_LOG_NOTICE, "  prng     = XORoshiro-256 (EXPERIMENTAL!)" );
             }
             else
             {
-                nwipe_log( NWIPE_LOG_NOTICE, "  prng     = Undefined" );
+                if( nwipe_options.prng == &nwipe_isaac )
+                {
+                    nwipe_log( NWIPE_LOG_NOTICE, "  prng     = Isaac" );
+                }
+                else
+                {
+                    if( nwipe_options.prng == &nwipe_isaac64 )
+                    {
+                        nwipe_log( NWIPE_LOG_NOTICE, "  prng     = Isaac64" );
+                    }
+                    else
+                    {
+                        nwipe_log( NWIPE_LOG_NOTICE, "  prng     = Undefined" );
+                    }
+                }
             }
         }
     }
@@ -557,7 +695,10 @@ void display_help()
     puts( "                          (default: last)" );
     puts( "                          off   - Do not verify" );
     puts( "                          last  - Verify after the last pass" );
-    puts( "                          all   - Verify every pass\n" );
+    puts( "                          all   - Verify every pass" );
+    puts( "                          " );
+    puts( "                          Please mind that HMG IS5 enhanced always verifies the" );
+    puts( "                          last (PRNG) pass regardless of this option.\n" );
     puts( "  -m, --method=METHOD     The wiping method. See man page for more details." );
     puts( "                          (default: dodshort)" );
     puts( "                          dod522022m / dod       - 7 pass DOD 5220.22-M method" );
@@ -568,9 +709,12 @@ void display_help()
     puts( "                          zero / quick           - Overwrite with zeros" );
     puts( "                          one                    - Overwrite with ones (0xFF)" );
     puts( "                          verify_zero            - Verifies disk is zero filled" );
-    puts( "                          verify_one             - Verifies disk is 0xFF filled\n" );
+    puts( "                          verify_one             - Verifies disk is 0xFF filled" );
+    puts( "                          is5enh                 - HMG IS5 enhanced\n" );
     puts( "  -l, --logfile=FILE      Filename to log to. Default is STDOUT\n" );
-    puts( "  -p, --prng=METHOD       PRNG option (mersenne|twister|isaac|isaac64)\n" );
+    puts( "  -P, --PDFreportpath=PATH Path to write PDF reports to. Default is \".\"" );
+    puts( "                           If set to \"noPDF\" no PDF reports are written.\n" );
+    puts( "  -p, --prng=METHOD       PRNG option (mersenne|twister|isaac|isaac64|add_lagg_fibonacci_prng)\n" );
     puts( "  -q, --quiet             Anonymize logs and the GUI by removing unique data, i.e." );
     puts( "                          serial numbers, LU WWN Device ID, and SMBIOS/DMI data" );
     puts( "                          XXXXXX = S/N exists, ????? = S/N not obtainable\n" );
