@@ -4,14 +4,15 @@
  * used by configure to dynamically assign those values
  * to documentation files.
  */
-const char* version_string = "0.37";
+const char* version_string = "0.38";
 const char* program_name = "nwipe";
 const char* author_name = "Martijn van Brummelen";
 const char* email_address = "git@brumit.nl";
 const char* years = "2024";
 const char* copyright = "Copyright Darik Horn <dajhorn-dban@vanadac.com>\n\
-Modifications to original dwipe Copyright Andy Beverley <andy@andybev.com>\n\
+Andy Beverley <andy@andybev.com> Martijn van Brummelen <martijn@brumit.nl>\n\
+Nick Law <shredos.eraser@gmail.com> (@PartialVolume) and others\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty; not even for MERCHANTABILITY or FITNESS\n\
 FOR A PARTICULAR PURPOSE.\n";
-const char* banner = "nwipe 0.37";
+const char* banner = "nwipe 0.38";
